/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="LimeGreen"><b>&bull;</b></font> Die Schnittstelle ILautsprecher bietet
 * Methoden zum Abspielen von Signalen, Tönen und Klängen mit dem Lautsprecher des Roboters.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public interface ILautsprecher
{
    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Liefert die Lautstärkeeinstellung des Lautsprechers.
     * Der mögliche Wertebereich liegt zwischen 0 (leise) und 100 (laut).
     * 
     * @return  die aktuell eingestellte Standard-Lautstärke
     */
    public int holeLautstaerke();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Setzt die Soll-Lautstärke des Lautsprechers.
     * Der erlaubte Wertebereich liegt zwischen 0 (leise) und 100 (laut).
     * 
     * @param   lautstaerke     die neue Standard-Lautstärke
     */
    public void setzeLautstaerke(int lautstaerke);

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Spielt einen Beep.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Beep.
     */
    public void spieleBeep();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Spielt einen doppelten Beep.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Doppel-Beep.
     */
    public void spieleBeepDoppelt();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Spielt eine Beepfolge.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende der Beepfolge.
     */
    public void spieleBeepfolge();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Spielt eine Beepfolge aufwärts.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende der Beepfolge.
     */
    public void spieleBeepfolgeAufwaerts();

    /**
     * <font color="LimeGreen"><b>&bull;</b></font> Spielt einen Buzz.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Buzz.
     */
    public void spieleBuzz();

    /**
     * <font color="Gold"><b>&bull;</b></font> Spielt einen Ton.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Tons.
     * 
     * @param   frequenz    die Tonfrequenz
     * @param   dauer       die Tondauer in Millisekunden
     */
    public void spieleTon(int frequenz, int dauer);

    /**
     * <font color="Gold"><b>&bull;</b></font> Spielt einen Ton.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Tons.
     * Die angegebene Lautstärke (Wertebereich zwischen 0 und 100) ist
     * relativ zur aktuell eingestellten Soll-Lautstärke.
     * 
     * @param   frequenz    die Tonfrequenz
     * @param   dauer       die Tondauer in Millisekunden
     * @param   lautstaerke die relative Lautstärke, in der der Ton gespielt wird.
     */
    public void spieleTon(int frequenz, int dauer, int lautstaerke);

    /**
     * <font color="Gold"><b>&bull;</b></font> Spielt einen Klang.
     * Die möglichen Klangcharakteristiken sind in der Aufzählungsklasse {@link EKlang} definiert.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende des Klangs.
     * Für den NXT- und EV3-Roboter ist die Klangmodulation in der aktuellen Version deaktiviert,
     * d.h. es wird ein einfacher Ton gespielt.
     * 
     * @param   klang       die zu verwendende Klangcharakteristik
     * @param   frequenz    die Klangfrequenz
     * @param   dauer       die Klangdauer in Millisekunden
     */
    public void spieleKlang(EKlang klang, int frequenz, int dauer);

    /**
     * <font color="Gold"><b>&bull;</b></font> Spielt eine Pause.
     * Die Ausführung des nächsten Programmbefehls erfolgt erst nach Ende der Pause.
     * Ein Aufruf dieser Methode führt zum gleichen Ergebnis wie ein Aufruf von {@link IUhr#warteZeit(int)}.
     * 
     * @param   dauer   die Pausendauer in Millisekunden
     */
    public void spielePause(int dauer);
}
