/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;


/**
 * <font color="Red"><b>&bull;</b></font> Die Schnittstelle IPCKonsole
 * dient der Kommunikation mit einem Anzeige-Konsolenfenster auf dem PC.
 * Typischerweise nutzt man diese Schnittstelle bei der Fehlersuche in
 * Roboterprogrammen, indem man während des Programmablaufs Statusmeldungen
 * an die PC-Konsole sendet. Gestartet wird das Konsolenfenster am einfachsten
 * aus der Entwicklungsumgebung heraus (in BlueJ über die Paketansicht:
 * Werkzeuge -> Roboter -> NXT-Konsole für NXT-Roboter bzw.
 * Werkzeuge -> Roboter -> EV3-Steuerung für EV3-Roboter).
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public interface IPCKonsole
{
    
    /**
     * <font color="Red"><b>&bull;</b></font> Baut eine Verbindung zur <b>(laufenden)</b> Anzeige-Konsolenanwendung auf dem PC auf.
     * Die Methode wartet, bis eine Verbindung zur Anzeige-Konsole auf dem PC zustande kommt.
     * Ohne Verbindung wird das Programm also nicht fortgesetzt. 
     */
    public void baueVerbindungAuf();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Trennt die Verbindung zur Anzeige-Konsole auf dem PC.
     * Eine bestehende Verbindung sollte vor dem Programmende auf dem Roboter wieder getrennt werden.
     */
    public void trenneVerbindung();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Ausgabe eines Textes ohne Zeilenumbruch in dem Anzeige-Konsolenfenster auf dem PC.
     * 
     * @param  text    der auszugebende Text
     */
    public void schreibe(String text);

    /**
     * <font color="Red"><b>&bull;</b></font> Ausgabe eines Textes mit Zeilenumbruch in dem Anzeige-Konsolenfenster auf dem PC.
     * 
     * @param  text    der auszugebende Text
     */
    public void schreibeZeile(String text);
}
