/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

/**
 * <font color="Red"><b>&bull;</b></font> Objekte, die die Schnittstelle
 * ISensorbeobachter unterstützen, können Ereignismeldungen eines Sensors empfangen.
 * Eine Ereignismeldung informiert über eine Zustandsänderung des Sensors,
 * d.h. über Änderungen des Messwerts oder dessen Änderungsrate.
 * Dazu muss das Beobachterobjekt beim Sensor als solches angemeldet werden
 * (siehe {@link ISensor#setzeBeobachter(ISensorbeobachter)}).
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public interface ISensorbeobachter
{
    /**
     * <font color="Red"><b>&bull;</b></font> Diese Methode wird vom Sensor an einem registierten
     * Sensorbeobachter aufgerufen, wenn sich der Sensorzustand (Messwert öder Änderunsrate) maßgeblich ändert.
     * Die Registrierung eines Sensorbeobachter erfolgt über die Methode {@link ISensor#setzeBeobachter(ISensorbeobachter)}.
     * Ob eine Änderung maßgeblich ist, wird über Filtertoleranzen festgelegt, die am Sensor eingestellt
     * werden können (siehe {@link ISensor#setzeFiltertoleranzen(float,float)}). Der Aufruf der Methode
     * {@link #registriereEreignis(ISensor,float,float,float,float)} erfolgt in einem eigenen Programmfaden,
     * der sich nur um die Benachrichtigung der Sensorbeobachter kümmert. Die Methode sollte so programmiert sein,
     * dass sie sehr schnell ausgeführt wird. Hier sollten nur die nötigen Informationen gesammelt werden,
     * damit der Roboter später aus dem Hauptprogrammfaden heraus angemessen reagieren kann.
     * Lesende Zugriffe auf Motoren und Sensoren sollten an dieser Stelle unproblematisch sein.
     * Der Programmfaden für die Benachrichtigung der Sensorbeobachter und der Hauptprogrammfaden des
     * Roboters dürfen sich ganz allgemein nicht gegenseitig in die Quere kommen. Der gleichzeitige
     * steuernde Zugriff auf Motoren und Sensoren oder der gleichzeitig schreibende Zugriff auf Variablen
     * führt z.B. sehr schnell zu unerwünschten Resultaten.
     * 
     * @param   sensor       aufrufender Sensor
     * @param   alterWert    alter Messwert
     * @param   neuerWert    neuer Messwert
     * @param   alteRate     alte Änderungsrate
     * @param   neueRate     neue Änderungsrate
     * @see     ISensor#setzeBeobachter(ISensorbeobachter)
     * @see     ISensor#loescheBeobachter()
     * @see     ISensor#setzeFiltertoleranzen(float,float)
     * @see     ISensor#holeFiltertoleranzMesswert()
     * @see     ISensor#holeFiltertoleranzAenderungsrate()
     * @see     ISensor#pruefeAufEreignisueberlauf()
     */
    void registriereEreignis(ISensor sensor, float alterWert, float neuerWert, float alteRate, float neueRate);
}
