/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.de;

import org.jabotics.robot.defs.*;

/**
 * Objekte vom Typ XDrucksensor repräsentieren die Drucksensoren eines NXT-Roboters.
 * Sie bietem dem Nutzer zur Ansteuerung die Schnittstelle {@link IDrucksensor}, die eine
 * Zusammenführung der Schnittstellen {@link ISensor} und {@link IDrucktaster} darstellt
 * (ein Drucksensor ist im Grunde auch nichts anderes als ein Drucktaster).
 * Damit steht auch die Schnittstelle {@link IEreignisMelder} zur Verfügung.
 * Der Konstruktor der Klasse ist "nicht öffentlich". Man kann deshalb
 * als Anwender selbst keine Objekte dieses Typs erzeugen.
 * Man verschafft sich Zugang zu einem neuen Drucksensor-Objekt, indem man die Methode
 * {@link IObjektfabrik#erzeugeDrucksensor(ESensorPort)} aufruft.
 * Man erhält Zugriff auf die einzige Objektfabrik-Instanz, indem man
 * die statische Methode {@link XObjektfabrik#holeObjektfabrik()} aufruft.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
class XTastsensor extends XSensor implements ITastsensor
{
    XTastsensor(ESensoranschluss anschluss)
    {
        super(anschluss,ESensorType.TOUCH_SENSOR);
    }

    XTastsensor(ETaster taster)
    {
        super(EButtonType.values()[taster.ordinal()]);
    }

    public boolean istGedrueckt()
    {
        return mSensor.isPressed();
    }
    
    public void warteBisGedrueckt()
    {
        mSensor.waitUntilPressed();
    }
    
    public void warteBisLosgelassen()
    {
        mSensor.waitUntilReleased();
    }
    
    public boolean warteBisGedrueckt(int maximale_wartezeit)
    {
        return mSensor.waitUntilPressed(maximale_wartezeit);
    }
    
    public boolean warteBisLosgelassen(int maximale_wartezeit)
    {
        return mSensor.waitUntilReleased(maximale_wartezeit);
    }

    public void warteBisEreignisGedrueckt()
    {
        mSensor.waitForPressEvent();
    }

    public void warteBisEreignisLosgelassen()
    {
        mSensor.waitForReleaseEvent();
    }

    public boolean warteBisEreignisGedrueckt(int maximale_wartezeit)
    {
        return mSensor.waitForPressEvent(maximale_wartezeit);
    }

    public boolean warteBisEreignisLosgelassen(int maximale_wartezeit)
    {
        return mSensor.waitForReleaseEvent(maximale_wartezeit);
    }
}
