/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;


/**
 * <font color="Red"><b>&bull;</b></font> The interface IPCConsole provides access to a console window on a connected PC.
 * Typically, this interface is being used for debugging purposes.
 * During program execution on the robot, logging output is sent to the console window on the PC.
 * When using the BlueJ integration, the console window can be started from the tools menu
 * (command <i>Robot -> NXT console viewer</i> for NXT robots or <i>Robot -> EV3 control viewer</i> for EV3 robots).
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public interface IPCConsole
{
    
    /**
     * <font color="Red"><b>&bull;</b></font> Establishes a connection to a <b>(running)</b> console application on the PC.
     */
    public void open();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Disconnects from the console application on the PC.
     * Before termination of the robot program, any connection to a console window should be terminated.
     */
    public void close();
    
    /**
     * <font color="Red"><b>&bull;</b></font> Text output without line break to a connected console window on a PC.
     * 
     * @param  text    text to be written to the console window on the PC
     */
    public void write(String text);

    /**
     * <font color="Red"><b>&bull;</b></font> Text output with line break to a connected console window on a PC.
     * 
     * @param  text    text to be written to the console window on the PC
     */
    public void writeln(String text);
}