/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.adapter.XAdapter;

/**
 * Objects of type XDisplay represent the display of the robot brick.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
class XDisplay implements IDisplay
{
    private final IDisplayAdapter mDisplay = XAdapter.getFactory().getDisplayAdapter();

    protected XDisplay()
    {
    }

    public void refresh()
    {
        mDisplay.refresh();
    }
    
    public void clear()
    {
        mDisplay.clear();
    }
    
    public void clearLine(int y_position)
    {
        mDisplay.clear(y_position);
    }
    
    public void clearCharacters(int x_position, int y_position, int number)
    {
        mDisplay.clear(x_position, y_position, number);
    }
    
    public void write(char character, int x_position, int y_position)
    {
        mDisplay.draw(character, x_position, y_position);
    }
    
    public void write(int integral_number, int x_position, int y_position)
    {
        mDisplay.draw(integral_number, x_position, y_position);
    }
    
    public void write(int integral_number, int x_position, int y_position, int digits)
    {
        mDisplay.draw(integral_number, x_position, y_position, digits);
    }
    
    public void write(float decimal_number, int x_position, int y_position)
    {
        mDisplay.draw(decimal_number, x_position, y_position);
    }
    
    public void write(String text, int x_position, int y_position)
    {
        mDisplay.draw(text, x_position, y_position);
    }
    
    public void write(String text, int x_position, int y_position, boolean hightlighted)
    {
        mDisplay.draw(text, x_position, y_position, hightlighted);
    }
}
