/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

import org.jabotics.robot.defs.*;
import org.jabotics.robot.core.CPilot;

/**
 * Objects of type XPilot represent an auto-pilot for controlling robot motions.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
class XPilot implements IPilot
{
    
    // Instanzvariablen - ersetzen Sie das folgende Beispiel mit Ihren Variablen
    private CPilot mPilot;

    /**
     * Konstruktor für Objekte der Klasse XPilot
     * Es werden Standardwerte für den Raddurchmesser und den Radabstand angenommen.
     */
    XPilot(org.jabotics.robot.en.EMotorPort portLeftMotor, org.jabotics.robot.en.EMotorPort portRightMotor)
    {
        mPilot = CPilot.getPilotSingleton(org.jabotics.robot.defs.EMotorPort.values()[portLeftMotor.ordinal()],org.jabotics.robot.defs.EMotorPort.values()[portRightMotor.ordinal()]);
    }

    /**
     * Konstruktor für Objekte der Klasse XPilot
     */
    XPilot(org.jabotics.robot.en.EMotorPort portLeftMotor, org.jabotics.robot.en.EMotorPort portRightMotor, float wheelDiameter, float wheelbase)
    {
        mPilot = CPilot.getPilotSingleton(org.jabotics.robot.defs.EMotorPort.values()[portLeftMotor.ordinal()],org.jabotics.robot.defs.EMotorPort.values()[portRightMotor.ordinal()],wheelDiameter,wheelbase);
    }

    @Override
    public float getWheelDiameter()
    {
        return mPilot.getWheelDiameter();
    }

    @Override
    public void setWheelDiameter(float diameter)
    {
        mPilot.setWheelDiameter(diameter);
    }

    @Override
    public float getWheelbase()
    {
        return mPilot.getWheelbase();
    }

    @Override
    public void setWheelbase(float wheelbase)
    {
        mPilot.setWheelbase(wheelbase);
    }

    @Override
    public float getSpeed()
    {
        return mPilot.getSpeed();
    }

    @Override
    public float getTargetSpeed()
    {
        return mPilot.getTargetSpeed();
    }

    @Override
    public void setTargetSpeed(float speed)
    {
        mPilot.setTargetSpeed(speed);
    }

    @Override
    public float getMaxSpeed()
    {
        return mPilot.getMaxSpeed();
    }

    @Override
    public void setTargetSpeedToMax()
    {
        mPilot.setTargetSpeedToMax();
    }

    @Override
    public float getTargetAcceleration()
    {
        return mPilot.getTargetAcceleration();
    }

    @Override
    public void setTargetAcceleration(float acceleration)
    {
        mPilot.setTargetAcceleration(acceleration);
    }

    @Override
    public float getMaxAcceleration()
    {
        return mPilot.getMaxAcceleration();
    }

    @Override
    public void setTargetAccelerationToMax()
    {
        mPilot.setTargetAccelerationToMax();
    }

    @Override
    public float getTachoCount()
    {
        return mPilot.getTachoCount();
    }
    
    @Override
    public float getOrientation()
    {
        return mPilot.getOrientation();
    }

    @Override
    public void resetCounters()
    {
        mPilot.resetCounters();
    }

    @Override
    public void moveStraight(EDirection direction)
    {
        mPilot.moveStraight(EDirection.FORWARD == direction);
    }

    @Override
    public void moveStraight(float distance, EWaitingMode waitingMode)
    {
        mPilot.moveStraight(distance, EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void moveCurvature(float curvature, EDirection direction)
    {
        mPilot.moveCurvature(curvature, EDirection.FORWARD == direction);
    }

    @Override
    public void moveRadius(float radius, EDirection direction)
    {
        mPilot.moveRadius(radius, EDirection.FORWARD == direction);
    }

    @Override
    public void moveRadius(float radius, float rotationAngle, EWaitingMode waitingMode)
    {
        mPilot.moveArc(D.DEGREE_TO_RAD_F * radius * rotationAngle, rotationAngle, EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void moveArc(float distance, float rotationAngle, EWaitingMode waitingMode)
    {
        mPilot.moveArc(distance, rotationAngle, EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void rotateByAngle(float angle, EWaitingMode waitingMode)
    {
        mPilot.rotateByAngle(angle, EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void rotateToOrientation(float orientation, EWaitingMode waitingMode)
    {
        mPilot.rotateToOrientation(orientation, EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public void waitUntilMotionCompleted()
    {
        mPilot.waitUntilCompleted();
    }
    
    @Override
    public void stopMotion(EWaitingMode waitingMode)
    {
        mPilot.stopMotion(EWaitingMode.WAIT == waitingMode);
    }

    @Override
    public boolean isMoving()
    {
        return mPilot.isMoving();
    }
}
