/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.Paint;
import java.awt.geom.AffineTransform;


/**
 * Tragen Sie hier eine Beschreibung der abstrakten Klasse BGraphicsElement ein.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */

public abstract class BGraphicsElement
{
    private AffineTransform mAtf = null;
    private AffineTransform mAtfInitial = null;
    private Paint mPaint = null;
    private Paint mPaintInitial = null;
    private Stroke mStroke = null;
    private Stroke mStrokeInitial = null;
    
    public synchronized void setTransform(float translationX, float translationY, float rotation)
    {
        mAtf = AffineTransform.getTranslateInstance(translationX, translationY);
        if (0f != rotation) mAtf.rotate(rotation); // => rotation will be "performed first", then translation
    }

    public synchronized void setPaint(Paint paint)
    {
        mPaint = paint;
    }

    public synchronized void setStroke(Stroke stroke)
    {
        mStroke = stroke;
    }

    final void applyContext(Graphics2D g)
    {
        mStrokeInitial = g.getStroke();
        mPaintInitial = g.getPaint();
        mAtfInitial = g.getTransform();

        if (null != mStroke) g.setStroke(mStroke);
        if (null != mPaint) g.setPaint(mPaint);
        if ((null != mAtf) && (mAtf.getType() != AffineTransform.TYPE_IDENTITY))
        {
            AffineTransform atf = new AffineTransform(mAtfInitial);
            atf.concatenate(mAtf); // => mAtf will be "performed first", then mAtfInitial
            g.setTransform(atf);
        }
    }

    final void resetContext(Graphics2D g)
    {
        g.setStroke(mStrokeInitial);
        g.setPaint(mPaintInitial);
        g.setTransform(mAtfInitial);
    }

    final synchronized void paint(Graphics2D g)
    {
        applyContext(g);
        paintInLocalContext(g);
        resetContext(g);
    }

    abstract protected void paintInLocalContext(Graphics2D g);
    
    /*public static void test()
    {
        AffineTransform aft;
        AffineTransform aft2;
        Point2D pt;
        Point2D pt2;
        
        pt = new Point2D.Double(10,10);
        aft = AffineTransform.getRotateInstance(Math.PI/6);
        pt2 = aft.transform(pt,null);
        aft.translate(10,10);          // here, the aft(!!!) is being "translated", that is:
        pt2 = aft.transform(pt,null);  // => the translation is performed first, then the rotation

        aft = AffineTransform.getRotateInstance(Math.PI/6);
        aft2 = AffineTransform.getTranslateInstance(10,10);
        aft.concatenate(aft2);          // in the resulting aft, the translation will be performed first
        pt2 = aft2.transform(pt,null);  // => the translation is performed first, then the rotation
    }*/
}
