/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.EButtonType;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.LineBorder;

/**
 * Beschreiben Sie hier die Klasse XBrickControl.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XBrickControl extends JPanel
{
    private static final XButton ENTER_BUTTON = XButton.getButton(EButtonType.ENTER);
    private static final XButton ESCAPE_BUTTON = XButton.getButton(EButtonType.ESCAPE);
    private static final XButton LEFT_BUTTON = XButton.getButton(EButtonType.LEFT);
    private static final XButton RIGHT_BUTTON = XButton.getButton(EButtonType.RIGHT);
    private static final int MARGIN_X = 4;
    private static final int MARGIN_Y = 1;


    private class ButtonHandler extends MouseAdapter
    {
        public void mouseEntered(MouseEvent e)
        {
            if (getCursor().getType() != Cursor.HAND_CURSOR)
            {
                getRootPane().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }
        }

        public void mouseExited(MouseEvent e)
        {
            if (getCursor().getType() != Cursor.DEFAULT_CURSOR)
            {
                getRootPane().setCursor(Cursor.getDefaultCursor());
            }
        }

        public void mousePressed(MouseEvent e)
        {
            Object source = e.getSource();
            if ( (source instanceof JButton) && (e.getButton() == MouseEvent.BUTTON1) )
            {
                JButton button = (JButton)source;
        
                if (button == mButtonEnter)
                {
                    ENTER_BUTTON.setPressed();
                    return;
                }
        
                if (button == mButtonEscape)
                {
                    ESCAPE_BUTTON.setPressed();
                    return;
                }
        
                if (button == mButtonLeft)
                {
                    LEFT_BUTTON.setPressed();
                    return;
                }
        
                if (button == mButtonRight)
                {
                    RIGHT_BUTTON.setPressed();
                    return;
                }
            }
        }

        public void mouseReleased(MouseEvent e)
        {
            Object source = e.getSource();
            if ( (source instanceof JButton) && (e.getButton() == MouseEvent.BUTTON1) )
            {
                JButton button = (JButton)source;
        
                if (button == mButtonEnter)
                {
                    ENTER_BUTTON.setReleased();
                    return;
                }
        
                if (button == mButtonEscape)
                {
                    ESCAPE_BUTTON.setReleased();
                    return;
                }
        
                if (button == mButtonLeft)
                {
                    LEFT_BUTTON.setReleased();
                    return;
                }
        
                if (button == mButtonRight)
                {
                    RIGHT_BUTTON.setReleased();
                    return;
                }
            }
        }
    }

    // instance variables:
    //private XEditorPane mDisplay;
    private JTextPane mDisplay;
    private JButton mButtonEnter;
    private JButton mButtonEscape;
    private JButton mButtonLeft;
    private JButton mButtonRight;


    /**
     * Konstruktor für Objekte der Klasse XBrickControl
     */
    public XBrickControl(boolean interactive)
    {
        super(new GridBagLayout());
        Dimension dim = new Dimension(210,285);
        this.setPreferredSize(dim);
        this.setMinimumSize(dim);
        this.setMaximumSize(dim);
        Color background = new Color(248,248,245);
        this.setBackground(background);
        this.setBorder(new LineBorder(Color.black,1));

        ClassLoader cl = this.getClass().getClassLoader();
        Icon iconEnter = new ImageIcon(cl.getResource("resources/images/BrickEnter.gif"));
        Icon iconEnterPressed = new ImageIcon(cl.getResource("resources/images/BrickEnterPressed.gif"));
        Icon iconEscape = new ImageIcon(cl.getResource("resources/images/BrickEscape.gif"));
        Icon iconEscapePressed = new ImageIcon(cl.getResource("resources/images/BrickEscapePressed.gif"));
        Icon iconLeft = new ImageIcon(cl.getResource("resources/images/BrickLeft.gif"));
        Icon iconLeftPressed = new ImageIcon(cl.getResource("resources/images/BrickLeftPressed.gif"));
        Icon iconRight = new ImageIcon(cl.getResource("resources/images/BrickRight.gif"));
        Icon iconRightPressed = new ImageIcon(cl.getResource("resources/images/BrickRightPressed.gif"));


        GridBagConstraints c;
        Insets noMargin = new Insets(0,0,0,0);
        //Insets smallMargin = new Insets(1,1,1,1);
        Insets stdMargin = new Insets(4,4,4,4);

        XDisplay display = XDisplay.getDisplay();
        mDisplay = new JTextPane(display.getDocument());
        mDisplay.setBackground(display.getBackground());
        mDisplay.setMargin(new Insets(MARGIN_Y,MARGIN_X,MARGIN_Y,MARGIN_X));
        Dimension dimDisplay = display.getDisplaySize();
        dimDisplay.width += 2*MARGIN_X;
        dimDisplay.height += 2*MARGIN_Y;
        JViewport viewport = new JViewport();  // use Viewport: mDisplay does not have to be sized => avoids fatal line break due to inaccurate font size calculation
        viewport.setPreferredSize(dimDisplay);
        viewport.setMinimumSize(dimDisplay);
        viewport.setMaximumSize(dimDisplay);
        viewport.setViewSize(dimDisplay);
        viewport.setExtentSize(dimDisplay);
        viewport.setView(mDisplay);
        viewport.setViewPosition(new Point(0,0));
        //dimDisplay.width += 50;
        //dimDisplay.height += 50;
        //mDisplay.setPreferredSize(dimDisplay);
        //mDisplay.setMinimumSize(dimDisplay);
        //mDisplay.setMaximumSize(dimDisplay);
        mDisplay.setEditable(false);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(4,4,24,4);
        this.add(viewport,c);

        JPanel paneButtons = new JPanel(new GridBagLayout());
        paneButtons.setBackground(background);
        c.gridx = 0;
        c.gridy = 1;
        c.fill = GridBagConstraints.NONE;
        c.insets = noMargin;
        this.add(paneButtons,c);

        mButtonEnter = new JButton(iconEnter);
        mButtonEnter.setPressedIcon(iconEnterPressed);
        mButtonEnter.setMargin(noMargin);
        mButtonEnter.setBorderPainted(false);
        mButtonEnter.setFocusPainted(false);
        mButtonEnter.setContentAreaFilled(false);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneButtons.add(mButtonEnter,c);

        mButtonEscape = new JButton(iconEscape);
        mButtonEscape.setPressedIcon(iconEscapePressed);
        mButtonEscape.setMargin(noMargin);
        mButtonEscape.setBorderPainted(false);
        mButtonEscape.setFocusPainted(false);
        mButtonEscape.setContentAreaFilled(false);
        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 1;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneButtons.add(mButtonEscape,c);

        mButtonLeft = new JButton(iconLeft);
        mButtonLeft.setPressedIcon(iconLeftPressed);
        mButtonLeft.setMargin(noMargin);
        mButtonLeft.setBorderPainted(false);
        mButtonLeft.setFocusPainted(false);
        mButtonLeft.setContentAreaFilled(false);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneButtons.add(mButtonLeft,c);

        mButtonRight = new JButton(iconRight);
        mButtonRight.setPressedIcon(iconRightPressed);
        mButtonRight.setMargin(noMargin);
        mButtonRight.setBorderPainted(false);
        mButtonRight.setFocusPainted(false);
        mButtonRight.setContentAreaFilled(false);
        c = new GridBagConstraints();
        c.gridx = 2;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = stdMargin;
        paneButtons.add(mButtonRight,c);

        if (interactive)
        {
            ButtonHandler buttonHandler = new ButtonHandler();

            mButtonEnter.addMouseListener(buttonHandler);
            mButtonEscape.addMouseListener(buttonHandler);
            mButtonLeft.addMouseListener(buttonHandler);
            mButtonRight.addMouseListener(buttonHandler);
        }
    }
}
