/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Beschreiben Sie hier die Klasse XOverviewPanel.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XOverviewPanel extends JPanel implements ActionListener, XPreferences.IListener
{
    // instance variables
    private XSceneView mSceneView;
    private XSceneOverview mSceneOV;
    private JButton mBtnPlus;
    private JButton mBtnMinus;
    private JProgressBar mBarScaling;

    public XOverviewPanel(XSimulationView simulationView, XSceneView sceneView)
    {
        super(new GridBagLayout());

        mSceneView = sceneView;
        mSceneOV = new XSceneOverview(sceneView);

        // Create icons
        ClassLoader cl = this.getClass().getClassLoader();
        Icon iconPlus = new ImageIcon(cl.getResource("resources/images/Plus.gif"));
        //Icon iconPlusPressed = new ImageIcon(cl.getResource("images/PlusPressed.gif"));
        Icon iconMinus = new ImageIcon(cl.getResource("resources/images/Minus.gif"));
        //Icon iconMinusPressed = new ImageIcon(cl.getResource("images/MinusPressed.gif"));

        GridBagConstraints c;
        Insets noMargin = new Insets(0,0,0,0);
        Dimension dimBar = new Dimension(30,141);

        mBtnPlus = new JButton(iconPlus);
        //mBtnPlus.setPressedIcon(iconPlusPressed);
        mBtnPlus.setMargin(noMargin);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(4,6,2,4);
        this.add(mBtnPlus,c);

        mBarScaling = new JProgressBar(SwingConstants.VERTICAL, XSceneView.MIN_SCALING, XSceneView.MAX_SCALING);
        mBarScaling.setValue(XPreferences.getReadInstance().getViewScaling());
        mBarScaling.setPreferredSize(dimBar);
        mBarScaling.setMinimumSize(dimBar);
        mBarScaling.setMaximumSize(dimBar);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 1;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(2,6,2,4);
        this.add(mBarScaling,c);

        mBtnMinus = new JButton(iconMinus);
        //mBtnMinus.setPressedIcon(iconMinusPressed);
        mBtnMinus.setMargin(noMargin);
        c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 2;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(2,6,2,4);
        this.add(mBtnMinus,c);

        c = new GridBagConstraints();
        c.gridx = 1;
        c.gridy = 0;
        c.gridheight = 3;
        c.fill = GridBagConstraints.NONE;
        c.insets = new Insets(4,4,2,6);
        this.add(mSceneOV,c);

        mBtnPlus.addActionListener(this);
        mBtnMinus.addActionListener(this);
        XPreferences.addListener(this);
    }

    public void updateGraphics()
    {
        mSceneOV.updateGraphics();
    }

    public void init()
    {
        mSceneOV.init();
    }

    public void actionPerformed(ActionEvent e)
    {
        int scaling = mBarScaling.getValue();

        if (e.getSource().equals(mBtnPlus))
        {
            if (scaling < XSceneView.MAX_SCALING)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                pw.setViewScaling(++scaling);
                pw.store();
            }
            return;
        }

        if (e.getSource().equals(mBtnMinus))
        {
            if (scaling > XSceneView.MIN_SCALING)
            {
                XPreferences.Write pw = XPreferences.getWriteInstance();
                pw.setViewScaling(--scaling);
                pw.store();
            }
            return;
        }
    }

    public void preferencesChanged(XPreferences.Read newPrefs)
    {
        int scaling = newPrefs.getViewScaling();
        if (mBarScaling.getValue() != scaling)
        {
            mBarScaling.setValue(scaling);
            mSceneView.setScaling(scaling);
        }
    }
}
