/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

import java.util.prefs.*;
import java.util.LinkedList;
import javax.swing.JFileChooser;
//import javax.swing.JOptionPane;
import java.io.*;

/**
 * Beschreiben Sie hier die Klasse XPreferences.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XPreferences
{
    // Klassenvariablen
    static final String AUDIO_MIXER_OFF = "___OFF___";
    static final String AUDIO_MIXER_DEFAULT = "___DEFAULT___";
    private static final String KEY_SIMULATION_USER = "jabotics.robot.simulation.user";
    private static final String KEY_SIMULATION_TEST = "jabotics.robot.simulation.test";
    private static final String KEY_SIMULATION_XML = "jabotics.robot.simulation.xml";
    private static final String KEY_VERSION = "version";
    private static final String KEY_VIEW_POS_X = "view.pos.x";
    private static final String KEY_VIEW_POS_Y = "view.pos.y";
    private static final String KEY_VIEW_SIZE_X = "view.size.x";
    private static final String KEY_VIEW_SIZE_Y = "view.size.y";
    private static final String KEY_VIEW_STATE = "view.state";
    private static final String KEY_VIEW_SCALING = "view.scaling";
    private static final String KEY_SIM_BREAKS_ENABLED = "sim.breaks.enabled";
    private static final String KEY_SIM_SUSPEND_ENABLED = "sim.suspend.enabled";
    private static final String KEY_SCENE_ID = "scene.id";
    private static final String KEY_ROBOT_START_X = "robot.start.x";
    private static final String KEY_ROBOT_START_Y = "robot.start.y";
    private static final String KEY_ROBOT_START_PHI = "robot.start.phi";
    private static final String KEY_AUDIO_INPUT_LEVEL = "audio.input.level";
    private static final String KEY_AUDIO_MIXER_IN = "audio.mixer.in";
    private static final String KEY_AUDIO_MIXER_OUT = "audio.mixer.out";
    private static String[] KEY_SENSOR_TYPE;
    private static String[] KEY_SENSOR_DIRECTION;
    static
    {
        KEY_SENSOR_TYPE = new String[D.SENSOR_PORT_NUMBER];
        KEY_SENSOR_DIRECTION = new String[D.SENSOR_PORT_NUMBER];
        for (int i = 0;  i < D.SENSOR_PORT_NUMBER;  ++i)
        {
            KEY_SENSOR_TYPE[i] = String.format("sensor%d.type",i+1);
            KEY_SENSOR_DIRECTION[i] = String.format("sensor%d.direction",i+1);
        }
    }

    private static final int DEF_VERSION = 2;
    private static final int DEF_VIEW_POS_X = 10;
    private static final int DEF_VIEW_POS_Y = 10;
    private static final int DEF_VIEW_SIZE_X = 780;
    private static final int DEF_VIEW_SIZE_Y = 580;
    private static final int DEF_VIEW_STATE = 0;
    private static final int DEF_VIEW_SCALING = 8;
    private static final int DEF_AUDIO_INPUT_LEVEL = 40;
    private static final String DEF_AUDIO_MIXER_IN = AUDIO_MIXER_OFF;
    private static final String DEF_AUDIO_MIXER_OUT = AUDIO_MIXER_DEFAULT;
    private static final boolean DEF_SIM_BREAKS_ENABLED = true;
    private static final boolean DEF_SIM_SUSPEND_ENABLED = true;

    private static final XPreferences DEF_PREFERENCES = new XPreferences();
    private static final LinkedList<IListener> mListeners = new LinkedList<IListener>();

    private static final Preferences sPrefsSimulationUser;
    private static final Preferences sPrefsSimulationTest;
    private static final Preferences sPrefsSimulationXML;
    private static volatile Preferences sPrefsCurrent;
    private static volatile XPreferences sPreferences;
    static
    {
        // Dirty hack to suppress output from Preferences implemention
        // when accessing root node

        PrintStream originalOut = System.out;
        PrintStream originalErr = System.err;

    	ByteArrayOutputStream baOut = new ByteArrayOutputStream();
        PrintStream psOut = new PrintStream(baOut);
        System.out.flush();
        System.setOut(psOut);

    	ByteArrayOutputStream baErr = new ByteArrayOutputStream();
        PrintStream psErr = new PrintStream(baErr);
        System.err.flush();
        System.setErr(psErr);

        Preferences prefsUser = Preferences.userRoot();
        sPrefsSimulationUser = prefsUser.node(KEY_SIMULATION_USER);
        sPrefsSimulationTest = prefsUser.node(KEY_SIMULATION_TEST);
        sPrefsSimulationXML = prefsUser.node(KEY_SIMULATION_XML);
        sPrefsCurrent = sPrefsSimulationUser;
        sPreferences = loadPreferences(sPrefsCurrent);

    	System.out.flush();
        System.setOut(originalOut);
        psOut.close();
        System.err.flush();
        System.setErr(originalErr);
        psErr.close();
    }

    // Instanzvariablen
    private int mVersion;
    private int mViewPosX;
    private int mViewPosY;
    private int mViewSizeX;
    private int mViewSizeY;
    private int mViewState;
    private int mViewScaling;
    private String mSceneID;
    private float mRobotStartX;
    private float mRobotStartY;
    private float mRobotStartPhi;
    private int mAudioInputLevel;
    private String mAudioMixerIn;
    private String mAudioMixerOut;
    private boolean mSimBreaksEnabled;
    private boolean mSimSuspendEnabled;
    private ESensorType[] mSensorTypes = new ESensorType[D.SENSOR_PORT_NUMBER];
    private ESensorDirection[] mSensorDirections = new ESensorDirection[D.SENSOR_PORT_NUMBER];

    static class Read
    {
        XPreferences mRef;

        private Read(XPreferences ref) {mRef = ref;}
        Read getReadInstance() {return new Read(mRef);}
        Write getWriteInstance() {return new Write(new XPreferences(mRef));}

        int getViewPosX() {return mRef.mViewPosX;}
        int getViewPosY() {return mRef.mViewPosY;}
        int getViewSizeX() {return mRef.mViewSizeX;}
        int getViewSizeY() {return mRef.mViewSizeY;}
        int getViewState() {return mRef.mViewState;}
        int getViewScaling() {return mRef.mViewScaling;}
        String getSceneID() {return mRef.mSceneID;}
        float getRobotStartX() {return mRef.mRobotStartX;}
        float getRobotStartY() {return mRef.mRobotStartY;}
        float getRobotStartPhi() {return mRef.mRobotStartPhi;}
        int getAudioInputLevel() {return mRef.mAudioInputLevel;}
        String getAudioMixerIn() {return mRef.mAudioMixerIn;}
        String getAudioMixerOut() {return mRef.mAudioMixerOut;}
        boolean getSimBreaksEnabled() {return mRef.mSimBreaksEnabled;}
        boolean getSimSuspendEnabled() {return mRef.mSimSuspendEnabled;}
        ESensorType getSensorType(ESensorPort port) {return mRef.mSensorTypes[port.ordinal()];}
        ESensorDirection getSensorDirection(ESensorPort port) {return mRef.mSensorDirections[port.ordinal()];}

    }

    static class Write extends Read
    {
        private Write(XPreferences ref) {super(ref);}
        void setToDefault() {mRef.setToDefault();}
        void store() {XPreferences.set(mRef);}

        void setViewPosX(int posX) {mRef.mViewPosX = posX;}
        void setViewPosY(int posY) {mRef.mViewPosY = posY;}
        void setViewSizeX(int sizeX) {mRef.mViewSizeX = sizeX;}
        void setViewSizeY(int sizeY) {mRef.mViewSizeY = sizeY;}
        void setViewState(int state) {mRef.mViewState = state;}
        void setViewScaling(int scaling) {mRef.mViewScaling = scaling;}
        void setSceneID(String sceneID) {mRef.mSceneID = sceneID;}
        void setRobotStartX(float x) {mRef.mRobotStartX = x;}
        void setRobotStartY(float y) {mRef.mRobotStartY = y;}
        void setRobotStartPhi(float phi) {mRef.mRobotStartPhi = phi;}
        void setAudioInputLevel(int level) {mRef.mAudioInputLevel = level;}
        void setAudioMixerIn(String mixer) {mRef.mAudioMixerIn = mixer;}
        void setAudioMixerOut(String mixer) {mRef.mAudioMixerOut = mixer;}
        void setSimBreaksEnabled(boolean enabled) {mRef.mSimBreaksEnabled = enabled;}
        void setSimSuspendEnabled(boolean enabled) {mRef.mSimSuspendEnabled = enabled;}
        void setSensorType(ESensorPort port, ESensorType type) {mRef.mSensorTypes[port.ordinal()] = type;}
        void setSensorDirection(ESensorPort port, ESensorDirection direction) {mRef.mSensorDirections[port.ordinal()] = direction;}
    }

    interface IListener
    {
        void preferencesChanged(XPreferences.Read newPreferences);
    }

    /**
     * Konstruktor für Objekte der Klasse PreferencesTest
     */
    private XPreferences()
    {
        setToDefault();
    }

    private XPreferences(XPreferences preferences)
    {
        mVersion = preferences.mVersion;
        mViewPosX = preferences.mViewPosX;
        mViewPosY = preferences.mViewPosY;
        mViewSizeX = preferences.mViewSizeX;
        mViewSizeY = preferences.mViewSizeY;
        mViewScaling = preferences.mViewScaling;
        mViewState = preferences.mViewState;
        mSceneID = preferences.mSceneID;
        mRobotStartX = preferences.mRobotStartX;
        mRobotStartY = preferences.mRobotStartY;
        mRobotStartPhi = preferences.mRobotStartPhi;
        mAudioInputLevel = preferences.mAudioInputLevel;
        mAudioMixerIn = preferences.mAudioMixerIn;
        mAudioMixerOut = preferences.mAudioMixerOut;
        mSimBreaksEnabled = preferences.mSimBreaksEnabled;
        mSimSuspendEnabled = preferences.mSimSuspendEnabled;
        for (int i = 0;  i < D.SENSOR_PORT_NUMBER;  ++i)
        {
            mSensorTypes[i] = preferences.mSensorTypes[i];
            mSensorDirections[i] = preferences.mSensorDirections[i];
        }
    }

    private void setToDefault()
    {
        XScene scene = XSimulationModel.getScene();
        mVersion = DEF_VERSION;
        mViewPosX = DEF_VIEW_POS_X;
        mViewPosY = DEF_VIEW_POS_Y;
        mViewSizeX = DEF_VIEW_SIZE_X;
        mViewSizeY = DEF_VIEW_SIZE_Y;
        mViewScaling = DEF_VIEW_SCALING;
        mViewState = DEF_VIEW_STATE;
        mSceneID = scene.getID();
        mRobotStartX = scene.getDefaultRobotStartX();
        mRobotStartY = scene.getDefaultRobotStartY();
        mRobotStartPhi = scene.getDefaultRobotStartPhi();
        mAudioInputLevel = DEF_AUDIO_INPUT_LEVEL;
        mAudioMixerIn = DEF_AUDIO_MIXER_IN;
        mAudioMixerOut = DEF_AUDIO_MIXER_OUT;
        mSimBreaksEnabled = DEF_SIM_BREAKS_ENABLED;
        mSimSuspendEnabled = DEF_SIM_SUSPEND_ENABLED;
        for (ESensorPort port : ESensorPort.values())
        {
            int i = port.ordinal();
            ESensorType type = BSensor.getDefaultSensorType(port);
            mSensorTypes[i] = type;
            mSensorDirections[i] = BSensor.getDefaultSensorDirection(type);
        }
    }

    static private XPreferences loadPreferences(Preferences prefs)
    {
        XPreferences preferences = new XPreferences(); // Default values

        if (DEF_VERSION == prefs.getInt(KEY_VERSION,DEF_VERSION))
        {
            preferences.mVersion = prefs.getInt(KEY_VERSION,preferences.mVersion);
            preferences.mViewPosX = prefs.getInt(KEY_VIEW_POS_X,preferences.mViewPosX);
            preferences.mViewPosY = prefs.getInt(KEY_VIEW_POS_Y,preferences.mViewPosY);
            preferences.mViewSizeX = prefs.getInt(KEY_VIEW_SIZE_X,preferences.mViewSizeX);
            preferences.mViewSizeY = prefs.getInt(KEY_VIEW_SIZE_Y,preferences.mViewSizeY);
            preferences.mViewState = prefs.getInt(KEY_VIEW_STATE,preferences.mViewState);
            preferences.mViewScaling = prefs.getInt(KEY_VIEW_SCALING,preferences.mViewScaling);
            if (preferences.mSceneID.equals(prefs.get(KEY_SCENE_ID,"")))
            {
                preferences.mRobotStartX = prefs.getFloat(KEY_ROBOT_START_X,preferences.mRobotStartX);
                preferences.mRobotStartY = prefs.getFloat(KEY_ROBOT_START_Y,preferences.mRobotStartY);
                preferences.mRobotStartPhi = prefs.getFloat(KEY_ROBOT_START_PHI,preferences.mRobotStartPhi);
            }
            preferences.mAudioInputLevel = prefs.getInt(KEY_AUDIO_INPUT_LEVEL,preferences.mAudioInputLevel);
            preferences.mAudioMixerIn = prefs.get(KEY_AUDIO_MIXER_IN,preferences.mAudioMixerIn);
            preferences.mAudioMixerOut = prefs.get(KEY_AUDIO_MIXER_OUT,preferences.mAudioMixerOut);
            preferences.mSimBreaksEnabled = prefs.getBoolean(KEY_SIM_BREAKS_ENABLED,preferences.mSimBreaksEnabled);
            preferences.mSimSuspendEnabled = prefs.getBoolean(KEY_SIM_SUSPEND_ENABLED,preferences.mSimSuspendEnabled);
            for (ESensorPort port : ESensorPort.values())
            {
                int i = port.ordinal();
                preferences.mSensorTypes[i] = ESensorType.values()[prefs.getInt(KEY_SENSOR_TYPE[i], preferences.mSensorTypes[i].ordinal())];
                preferences.mSensorDirections[i] = ESensorDirection.values()[prefs.getInt(KEY_SENSOR_DIRECTION[i], preferences.mSensorDirections[i].ordinal())];
            }
        }

        return preferences;
    }

    private void store(Preferences prefs)
    {
        try
        {
            prefs.clear();

            prefs.putInt(KEY_VERSION,mVersion);
            prefs.putInt(KEY_VIEW_POS_X,mViewPosX);
            prefs.putInt(KEY_VIEW_POS_Y,mViewPosY);
            prefs.putInt(KEY_VIEW_SIZE_X,mViewSizeX);
            prefs.putInt(KEY_VIEW_SIZE_Y,mViewSizeY);
            prefs.putInt(KEY_VIEW_STATE,mViewState);
            prefs.putInt(KEY_VIEW_SCALING,mViewScaling);
            prefs.put(KEY_SCENE_ID,mSceneID);
            prefs.putFloat(KEY_ROBOT_START_X,mRobotStartX);
            prefs.putFloat(KEY_ROBOT_START_Y,mRobotStartY);
            prefs.putFloat(KEY_ROBOT_START_PHI,mRobotStartPhi);
            prefs.putInt(KEY_AUDIO_INPUT_LEVEL,mAudioInputLevel);
            prefs.put(KEY_AUDIO_MIXER_IN,mAudioMixerIn);
            prefs.put(KEY_AUDIO_MIXER_OUT,mAudioMixerOut);
            prefs.putBoolean(KEY_SIM_BREAKS_ENABLED,mSimBreaksEnabled);
            prefs.putBoolean(KEY_SIM_SUSPEND_ENABLED,mSimSuspendEnabled);
            for (int i = 0;  i < D.SENSOR_PORT_NUMBER;  ++i)
            {
                prefs.putInt(KEY_SENSOR_TYPE[i], mSensorTypes[i].ordinal());
                prefs.putInt(KEY_SENSOR_DIRECTION[i], mSensorDirections[i].ordinal());
            }

            prefs.flush();
        }
        catch (BackingStoreException bse)
        {
            displayErrorMessage(XMsg.PREFS_WRITE_ERROR, "BackingStoreException", bse);
        }
    }
    
    static XPreferences.Read getReadInstance()
    {
        return new XPreferences.Read(sPreferences);
    }

    static XPreferences.Write getWriteInstance()
    {
        return new XPreferences.Write(new XPreferences(sPreferences));
    }

    synchronized static private void set(XPreferences preferences)
    {
        sPreferences = new XPreferences(preferences);
        notifyListeners();
    }

    synchronized static void reset()
    {
        sPreferences = DEF_PREFERENCES;        
        notifyListeners();
    }

    synchronized static void flush()
    {
        sPreferences.store(sPrefsCurrent);
    }

    synchronized static void exportToXML()
    {
        JFileChooser chooser = new JFileChooser();
        int result = chooser.showSaveDialog(null);
        if (JFileChooser.APPROVE_OPTION == result) exportToXML(chooser.getSelectedFile());
    }

    synchronized static void exportToXML(String pathname)
    {
        try
        {
            File file = new File(pathname);
            if (file.createNewFile() || (file.isFile() && file.canWrite())) exportToXML(file);
        }
        catch (IOException ioe)
        {
            displayErrorMessage(XMsg.PREFS_EXPORT_ERROR, "IOException", ioe);
        }
    }

    synchronized static void exportToXML(File file)
    {
        try
        {
            FileOutputStream fos = new FileOutputStream(file);
            sPreferences.store(sPrefsSimulationXML);
            sPrefsSimulationXML.exportSubtree(fos);
            fos.flush();
            fos.close();
        }
        catch (IOException ioe)
        {
            displayErrorMessage(XMsg.PREFS_EXPORT_ERROR, "IOException", ioe);
        }
        catch (BackingStoreException bse)
        {
            displayErrorMessage(XMsg.PREFS_EXPORT_ERROR, "BackingStoreException", bse);
        }
    }

    synchronized static void importFromXML()
    {
        JFileChooser chooser = new JFileChooser();
        int result = chooser.showOpenDialog(null);
        if (JFileChooser.APPROVE_OPTION == result) importFromXML(chooser.getSelectedFile());
    }

    synchronized static void importFromXML(String pathname)
    {
        File file = new File(pathname);
        if (file.isFile() && file.canRead()) importFromXML(file);
    }

    synchronized static void importFromXML(File file)
    {
        XPreferences preferences;
        try
        {
            sPrefsSimulationXML.clear();
            FileInputStream fis = new FileInputStream(file);
            Preferences.importPreferences(fis);
            fis.close();
            preferences = loadPreferences(sPrefsSimulationXML);
        }
        catch (BackingStoreException bse)
        {
            displayErrorMessage(XMsg.PREFS_IMPORT_ERROR, "BackingStoreException", bse);
            return;
        }
        catch (IOException ioe)
        {
            displayErrorMessage(XMsg.PREFS_IMPORT_ERROR, "IOException", ioe);
            return;
        }
        catch (InvalidPreferencesFormatException ipfe)
        {
            displayErrorMessage(XMsg.PREFS_IMPORT_ERROR, "InvalidPreferencesFormatException", ipfe);
            return;
        }

        if (DEF_VERSION != preferences.mVersion)
        {
            displayErrorMessage(XMsg.PREFS_IMPORT_ERROR + "\n" + XMsg.ERROR_TYPE + " " + XMsg.WRONG_FILE_VERSION);
        }
        else
        {
            sPreferences = preferences;
            sPreferences.store(sPrefsCurrent);
            notifyListeners();
        }
    }

    static public void enableTestMode(boolean enabled)
    {
        sPrefsCurrent = enabled ? sPrefsSimulationTest : sPrefsSimulationUser;
    }

    synchronized static public void addListener(IListener listener)
    {
        if (!mListeners.contains(listener)) mListeners.add(listener);
    }

    synchronized static public boolean removeListener(IListener listener)
    {
        return mListeners.remove(listener);
    }

    synchronized static public void removeAllListeners()
    {
        mListeners.clear();
    }

    static private void notifyListeners()
    {
        for (IListener listener : mListeners)
        {
            listener.preferencesChanged(new XPreferences.Read(sPreferences));
        }
    }

    static private void displayErrorMessage(String message)
    {
        //JOptionPane.showMessageDialog(null,message);
        System.err.println(message);
    }

    static private void displayErrorMessage(String message, Exception e)
    {
        String em = e.getMessage();
        if (null != em)  message = message + "\n" + em;
        displayErrorMessage(message);
    }

    static private void displayErrorMessage(String message, String error_type, Exception e)
    {
        if (null != error_type) message = message + "\n" + XMsg.ERROR_TYPE + " " + error_type;
        displayErrorMessage(message, e);
    }

    /*static public void test()
    {
        showPreferences("1");
        change();
        showPreferences("2");
        exportToXML();
        reset();
        showPreferences("3");
        importFromXML();
        showPreferences("4");
    }

    static private void change()
    {
        XPreferences p = XPreferences.get();
        p.mRobotStartX += 23.3;
        p.mRobotStartY += 12.4;
        p.mRobotStartPhi -= 3.1415926;
        if (p.mSensorTypes[2] == ESensorType.SOUND_SENSOR)
        {
            p.mSensorTypes[2] = ESensorType.LIGHT_SENSOR;
            p.mSensorDirections[2] = BSensor.getDefaultSensorDirection(ESensorType.LIGHT_SENSOR);
        }
        else
        {
            p.mSensorTypes[2] = ESensorType.SOUND_SENSOR;
            p.mSensorDirections[2] = BSensor.getDefaultSensorDirection(ESensorType.SOUND_SENSOR);
        }
        XPreferences.set(p);
    }

    static private void showPreferences(String title)
    {
        StringBuilder sb = new StringBuilder();
        
        sb.append(KEY_SIMULATION_USER);
        sb.append('\n');

        sb.append(KEY_VERSION);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VERSION,DEF_VERSION));
        sb.append('\n');

        sb.append(KEY_VIEW_POS_X);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VIEW_POSX,DEF_VIEW_POS_X));
        sb.append('\n');

        sb.append(KEY_VIEW_POS_Y);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VIEW_POSY,DEF_VIEW_POS_Y));
        sb.append('\n');

        sb.append(KEY_VIEW_SIZE_X);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VIEW_SIZEX,DEF_VIEW_SIZE_X));
        sb.append('\n');

        sb.append(KEY_VIEW_SIZE_Y);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VIEW_SIZEY,DEF_VIEW_SIZE_Y));
        sb.append('\n');

        sb.append(KEY_VIEW_SCALING);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_VIEW_SCALING,DEF_VIEW_SCALING));
        sb.append('\n');

        sb.append(KEY_ROBOT_START_X);
        sb.append(": ");
        sb.append(sPrefsCurrent.getDouble(KEY_ROBOT_STARTX,DEF_ROBOT_START_X));
        sb.append('\n');

        sb.append(KEY_ROBOT_START_Y);
        sb.append(": ");
        sb.append(sPrefsCurrent.getDouble(KEY_ROBOT_STARTY,DEF_ROBOT_START_Y));
        sb.append('\n');

        sb.append(KEY_ROBOT_START_PHI);
        sb.append(": ");
        sb.append(sPrefsCurrent.getDouble(KEY_ROBOT_STARTPHI,DEF_ROBOT_START_PHI));
        sb.append('\n');

        sb.append(KEY_AUDIO_INPUT_LEVEL);
        sb.append(": ");
        sb.append(sPrefsCurrent.getInt(KEY_AUDIO_INPUT_LEVEL,DEF_AUDIO_INPUT_LEVEL));
        sb.append('\n');

        for (ESensorPort port : ESensorPort.values())
        {
            int i = port.ordinal();

            String keyType = KEY_SENSOR_TYPE[i];
            sb.append(keyType);
            sb.append(": ");
            ESensorType type = ESensorType.values()[sPrefsCurrent.getInt(keyType, BSensor.getDefaultSensorType(port).ordinal())];
            sb.append(type.ordinal());
            sb.append('\n');
    
            String keyDir = KEY_SENSOR_DIR[i];
            sb.append(keyDir);
            sb.append(": ");
            sb.append(sPrefsCurrent.getInt(keyDir, BSensor.getDefaultSensorDirection(type).ordinal()));
            sb.append('\n');
        }

        JFrame frame = new JFrame(title);
        JTextArea area = new JTextArea(sb.toString());
        frame.add(area);
        frame.pack();
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        frame.setVisible(true);
    }*/
}