/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

/**
 * Beschreiben Sie hier die Klasse XSoundSensor.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XSoundSensor extends BSensor
{
    // Klassenvariablen:
    private static final XSoundSensor sSoundSensor = new XSoundSensor();

    static public XSoundSensor getSoundSensor(ESensorDirection direction)
    {
        switch (direction)
        {
            default:  return sSoundSensor;
        }
    }

    // Instanzvariablen:
    double mLastUpdateTime = -1.;
    float mValue = 0f;
    XSoundscape mSoundscape = XSoundscape.getSoundscape();


    private XSoundSensor()
    {
        super();
        mDirection = ESensorDirection.NONE;
        buildGraphics();
    }

    private void buildGraphics()
    {
    }

    @Override
    public ESensorType getType()
    {
        return ESensorType.SOUND_SENSOR;
    }

    @Override
    public synchronized float getValue(XSensorRef ref)
    {
        if (XSimulationModel.sTime - mLastUpdateTime > 0.005)
        {
            mValue = mSoundscape.getSoundLevel();
            if (mValue < 0)
            {
                mValue = 0f;
            }
            else if (mValue > 100f)
            {
                mValue = 100f;
            }
            mLastUpdateTime = XSimulationModel.sTime;
        }

        return mValue;
    }

    /*protected void setValue(int value)
    {
        if ((0 <= value) && (100 >= value))  mValue = value;
    }*/

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitSoundSensor(this);
    }

    @Override
    public ISessionData resetSession()
    {
        mLastUpdateTime = -1.;
        mValue = 0;
        return this;
    }
}
