/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.defs.*;

/**
 * XSpeaker
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class XSpeaker extends BModelElement implements ISoundAdapter
{
    // class variables:
    static private XSpeaker sSpeaker = null;
    static private final int MAX_VOLUME = 100;
    static private final float MAX_VOLUME_F = (float)MAX_VOLUME;
    static private final int C2 = 523;
    //static private final int[] PIANO = new int[]{4, 25, 500, 7000, 5};
    //static private final int[] FLUTE = new int[]{10, 25, 2000, 1000, 25};
    //static private final int[] XYLOPHONE = new int[]{1, 8, 3000, 5000, 5};

    // instance variables:
    private volatile float mMasterVolumeRel = 0.5f; // relative value between 0 and 1

    

    static XSpeaker getSpeaker()
    {
        if (null == sSpeaker) sSpeaker = new XSpeaker();

        return sSpeaker;
    }

    static boolean initialized()
    {
        return (null != sSpeaker);
    }

    private XSpeaker()
    {
    }

    @Override
    public void manageVisitor(BModelVisitor visitor)
    {
        visitor.visitSpeaker(this);
    }

    @Override
    public int getVolume()
    {
    return Math.round(mMasterVolumeRel * MAX_VOLUME_F);
    }

    @Override
    public void setVolume(int volume)
    {
        float rel_volume = (float)volume / MAX_VOLUME_F;
        mMasterVolumeRel = (rel_volume < 0f) ? 0f : ((rel_volume > 1f) ? 1f : rel_volume);
    }

    @Override
    public void beep()
    {
        XAudioSystem.tone(600, 200, mMasterVolumeRel);
    }

    @Override
    public void twoBeeps()
    {
        // Compare implementation of lejos.nxt.Sound.twoBeeps()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
        XAudioSystem.tone(600, 150, mMasterVolumeRel);
        XTimer.msDelayRobot(50);
        XAudioSystem.tone(600, 150, mMasterVolumeRel);
    }

    @Override
    public void beepSequence()
    {
        // Compare implementation of lejos.nxt.Sound.beepSequence()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
        for (int i = 7; i > 3; --i)
        {
            XAudioSystem.tone(C2 * i / 4, 100, mMasterVolumeRel);
        }
    }

    @Override
    public void beepSequenceUp()
    {
        // Compare implementation of lejos.nxt.Sound.beepSequenceUp()
        // and lejos.nxt.Sound.systemSound(boolean aQueued, int aCode).
        for (int i = 4; i < 8; ++i)
        {
            XAudioSystem.tone(C2 * i / 4, 100, mMasterVolumeRel);
        }
    }

    @Override
    public void buzz()
    {
        XAudioSystem.modulatedTone(450, 50, 1f, 500, mMasterVolumeRel);
    }

    @Override
    public void tone(int frequency, int length)
    {
        XAudioSystem.tone(frequency, length, mMasterVolumeRel);
    }

    @Override
    public void tone(int frequency, int length, int volume)
    {
        if (volume < 0)
        {
            volume = 0;
        }
        else if (volume > MAX_VOLUME)
        {
            volume = MAX_VOLUME;
        }

        XAudioSystem.tone(frequency, length, (mMasterVolumeRel*volume)/MAX_VOLUME_F);
    }

    @Override
    public void note(ESound instrument, int frequency, int length)
    {
        switch (instrument)
        {
            case PIANO:
            {
                XAudioSystem.modulatedTone(frequency, 2*frequency, 0.04f, length, mMasterVolumeRel, 5, 2000, 0f);
                break;
            }
            case FLUTE:
            {
                XAudioSystem.modulatedTone(frequency, frequency / 70 , 0.1f, length, mMasterVolumeRel, 10, 200, 0.5f);
                break;
            }
            default:
            {
                // XYLOPHONE
                XAudioSystem.tone(frequency, length, mMasterVolumeRel, 2, 300, 0f);
                break;
            }
        }
    }

    @Override
    public void pause(int length)
    {
        if (length > 0) XTimer.msDelayRobot(length);
    }

    static public void testTone()
    {
        try
        {
            Class.forName("org.jabotics.robot.sim.XAudioSystem"); // XAudioSystem needs to be initialized first
        }
        catch (Exception e) {}
        XSpeaker speaker = getSpeaker();
        XSimulation.resumeSimulation();

        speaker.beep();
        speaker.pause(1000);
        speaker.twoBeeps();
        speaker.pause(1000);
        speaker.buzz();
        speaker.pause(1000);
        speaker.beepSequence();
        speaker.pause(1000);
        speaker.beepSequenceUp();
        speaker.pause(1000);
        speaker.tone(440,1000);
        speaker.pause(1000);
        speaker.note(ESound.PIANO,440,1000);
        speaker.pause(1000);
        speaker.note(ESound.FLUTE,440,1000);
        speaker.pause(1000);
        speaker.note(ESound.XYLOPHONE,440,1000);
        speaker.pause(1000);
        XAudioSystem.playRobotSignal();
    }
}
