/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/

package org.jabotics.test.robot;


import static org.jabotics.test.Assert.*;

import org.jabotics.robot.de.*;

/**
 * Beschreibe hier die Klasse TestDispatcher.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class TestDispatcher extends BTestRoboter implements ISensorbeobachter
{
    long mStartzeitpunkt;
    long mEndzeitpunkt;
    float sMin; // minimaler Abstand
    float wMin; // zugehöriger Ausrichtungswinkel;
    float hMax; // maximale Helligkeit
    float wMax; // zugehöriger Ausrichtungswinkel;

    public static void main(String[] args)
    {
        TestDispatcher roboter = new TestDispatcher();
        roboter.run();
    }

    /**
     * Hier können bei Bedarf zusätzlich Initialisierungen einer mit dem new-Operator
     * erzeugten Roboterinstanz vorgenommen werden.
     */
    public TestDispatcher()
    {
        // In diesem Falle müssen die Instanzvariablen nicht initialisiert werden.
        // Die Initialisierung der Variablen sMin und wMin erfolgt an geeigneter
        // Stelle in der run()-Methode.
    }

    @Override
    public void run()
    {
        mStartzeitpunkt = uhr.holeSystemzeit();

        // Ersetze hier den Beispiel-Programmcode durch deine eigene Robotersteuerung.

        lautsprecher.spieleBeep();
        //schallsensor.warteBisGrenzwertUeberschritten(60);
        uhr.warteZeit(2000); // Zeit, um aus dem Weg zu gehen

        pilot.setzeZaehlerstaendeZurueck();
        pilot.setzeSollgeschwindigkeit(5);
        sMin = 10000f;                                  // Startwert für den kleinsten bisher gefundenen Abstand
        wMin = 0f;                                      // Startwert für den zugehörigen Ausrichtungswinkel (=0)
        hMax = 0f;                                      // Startwert für die maximale Helligkeit
        wMax = 0f;                                      // Startwert zugehöriger Ausrichtungswinkel;
        ultraschallsensor.setzeBeobachter(this);        // das Roboterobjekt wird als Beobachter angemeldet
        lichtsensor.setzeBeobachter(this);              // das Roboterobjekt wird als Beobachter angemeldet
        pilot.dreheZuAusrichtungswinkel(360,ABWARTEN);  // Roboter dreht sich einmal um 360°
                                                        // Währenddessen meldet der Sensor Änderungsereignisse
        ultraschallsensor.loescheBeobachter();          // Überwachung des Sensors wird beendet
        lichtsensor.loescheBeobachter();                // Überwachung des Sensors wird beendet
        //if (wMin < 180f) wMin = wMin + 360f;            // eine kleine Optimierung für folgenden Schritt
        pilot.dreheZuAusrichtungswinkel(wMax,ABWARTEN); // Hindrehen zur Richtung mit der größten Helligkeit.
        lautsprecher.spieleBeepfolgeAufwaerts();
        pilot.dreheZuAusrichtungswinkel(wMin,ABWARTEN); // Hindrehen zur Richtung mit dem kleinsten registrierten Abstand.
        pilot.setzeSollgeschwindigkeit(10);
        pilot.fahreGerade(sMin - 30, ABWARTEN);         // Roboter bleibt rechtzeitig vor Hindernis stehen.
                                                        // HINWEIS:
                                                        // Der kleinste Abstand zum nächsten Hindernis wird vom Roboter nicht richtig
                                                        // ermittelt, wenn dieser über der Reichweite des Ultraschallsensors liegt.
                                                        // Ggf. ist der Roboter beim Testen des Programms so aufzustellen,
                                                        // dass das nächstegelegene Hindernis nicht weiter als 2m entfernt ist.
        lautsprecher.spieleBeepfolge();
        monitor.loescheAlles();
        monitor.schreibe("Abstand in cm:", 0, 0);
        monitor.schreibe(sMin, 0, 1);
        monitor.schreibe("W:", 0, 2);
        monitor.schreibe(wMin, 3, 2);
        monitor.schreibe("H:", 0, 3);
        monitor.schreibe(hMax, 3, 3);
        monitor.schreibe("W:", 0, 4);
        monitor.schreibe(wMax, 3, 4);
        monitor.schreibe("T:", 0, 5);
        monitor.schreibe(pilot.holeTachostand(), 3, 5);
        //monitor.schreibe("Endetaste", 0, 6);
        //monitor.schreibe("druecken!", 5, 7);
        //endetaste.warteBisEreignisLosgelassen();
        uhr.warteZeit(2000);

        mEndzeitpunkt = uhr.holeSystemzeit();
    }

    @Override
    public void registriereEreignis(ISensor sensor, float alterWert, float neuerWert, float alteAenderungsrate, float neueAenderungsrate)
    {
        if (ultraschallsensor.equals(sensor))
        {
            // Hierher kommt der Programmcode, der auf eine Änderung des Messwerts des Ultraschallsensors reagiert.
            if (neuerWert < sMin)
            {
                sMin = neuerWert;
                wMin = pilot.holeAusrichtungswinkel();
            }
            return;
        }
        if (lichtsensor.equals(sensor))
        {
            // Hierher kommt der Programmcode, der auf eine Änderung des Messwerts des Ultraschallsensors reagiert.
            if (neuerWert > hMax)
            {
                hMax = neuerWert;
                wMax = pilot.holeAusrichtungswinkel();
            }
            return;
        }
    }

    @Override
    public void checkAsserts()
    {
        /*System.out.println("TestDispatcher:");
        System.out.println("Start:   " + mStartzeitpunkt);
        System.out.println("Ende:    " + mEndzeitpunkt);
        System.out.println("Dauer:   " + (mEndzeitpunkt - mStartzeitpunkt));
        System.out.println("Strecke: " + pilot.holeTachostand());
        System.out.println("Winkel:  " + pilot.holeAusrichtungswinkel());
        System.out.println("sMin: " + sMin);
        System.out.println("wMin: " + wMin);
        System.out.println("hMax: " + hMax);
        System.out.println("wMax: " + wMax);*/

        assertEquals("Total distance does not meet expection.", 112f, pilot.holeTachostand(), 10f);
        assertEquals("Total robot rotation does not meet expection.", 205f, pilot.holeAusrichtungswinkel(), 20f);
        assertEquals("Execution time does not meet expection.", 27600, (mEndzeitpunkt - mStartzeitpunkt), 1500);
        assertEquals("Min. distance does not meet expection.", 142f, sMin, 5f);
        assertEquals("Angle for min. distance does not meet expection.", 206f, wMin, 8f);
        assertEquals("Max. luminosity does not meet expection.", 99f, hMax, 2f);
        assertEquals("Angle for max. luminosity does not meet expection.", 179f, wMax, 8f);
    }
}
