/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/

package org.jabotics.test.robot;


import static org.jabotics.test.Assert.*;

/**
 * Beschreibe hier die Klasse TestFollowMe.
 * 
 * @author  Frank Gollnik
 * @version 1.1.2
 */
public class TestFollowMe extends BTestRoboter
{
    long mStartzeitpunkt;
    long mEndzeitpunkt;

    public static void main(String[] args)
    {
        TestFollowMe roboter = new TestFollowMe();
        roboter.run();
    }

    public TestFollowMe()
    {
    }

    @Override
    public void run()
    {
        mStartzeitpunkt = uhr.holeSystemzeit();

        uhr.warteZeit(2000);

        lichtsensor.schalteLichtEin();  // Die Lichtquelle des Sensors soll den Boden beleuchten
        uhr.warteZeit(100);

        pilot.setzeZaehlerstaendeZurueck();  // Zurücksetzen des Tachostands
        pilot.setzeSollgeschwindigkeit(10);  // mäßig große Sollgeschwindigkeit von 10cm/s
        lichtsensor.setzeTraegheit(100);  // Der Sensor soll nicht zu schnell auf Helligkeitsschwankungen (z.B. durch Schmutz) reagieren.
                                          // Die angegebene Zeit von 100 ms ist der Mittelungszeitraum.
        int helligkeitsgrenze = 50;  // Helligkeitsgrenze für das Erkennen der Kante der Linie.
                                     // Der Wert kann bei Bedarf an die Gegebenheiten angepasst werden.
                                     // Wert 0 entspricht sehr dunkel, 100 entspricht sehr hell.
        int radius = 10;  // Kurvenradius in cm für die folgenden Kurvenfahrten

        do   // Beginn einer Schleife
        {
            pilot.fahreRadius(radius,VORWAERTS);   // Kurvenfahrt nach links vorwärts mit dem angegebenen Kurvenradius.
            lichtsensor.warteBisGrenzwertUeberschritten(helligkeitsgrenze);  // Fahrt nach links fortsetzen, solange Boden dunkel
            pilot.fahreRadius(-radius,VORWAERTS);   // Kurvenfahrt nach rechts vorwärts mit dem angegebenen Kurvenradius.
            lichtsensor.warteBisGrenzwertUnterschritten(helligkeitsgrenze);  // Fahrt nach rechts fortsetzen, solange Boden hell
        }
        while (pilot.holeTachostand() < 1000);   // Der Tachostand (in cm) wird überprüft.
        pilot.halteAn(ABWARTEN);

        lichtsensor.schalteLichtAus();

        //Die gefahrenen Strecke soll noch angezeigt werden:
        float strecke = pilot.holeTachostand(); // Hier wird der Tachostand ausgelesen und in der neuen Fließkomma-Variablen strecke gespeichert.
        monitor.loescheAlles();
        monitor.schreibe("Strecke in cm:", 0, 3);
        monitor.schreibe(strecke, 0, 4);

        lautsprecher.spieleBeepfolgeAufwaerts();

        uhr.warteZeit(2000);

        mEndzeitpunkt = uhr.holeSystemzeit();
    }

    public void trailer()
    {
        uhr.warteZeit(2000);

        boolean invertiert = false;
        for (int i = 0;  i < 8;  ++i)
        {
            invertiert = !invertiert;
            monitor.schreibe("Folge mir!", 3, 2, invertiert);
            monitor.schreibe("Follow me!", 3, 4, invertiert);
            uhr.warteZeit(500);
        }
    }

    @Override
    public void checkAsserts()
    {
        /*System.out.println("TestFollowMe:");
        System.out.println("Start:   " + mStartzeitpunkt);
        System.out.println("Ende:    " + mEndzeitpunkt);
        System.out.println("Dauer:   " + (mEndzeitpunkt - mStartzeitpunkt));
        System.out.println("Strecke: " + pilot.holeTachostand());
        System.out.println("Winkel:  " + pilot.holeAusrichtungswinkel());*/

        assertEquals("Total distance does not meet expection.", 1004f, pilot.holeTachostand(), 10f);
        assertEquals("Total robot rotation does not meet expection.", -1047f, pilot.holeAusrichtungswinkel(), 20f);
        assertEquals("Execution time does not meet expection.", 161500, (mEndzeitpunkt - mStartzeitpunkt), 2000);
    }
}
